package jdk.net;

import jdk.Exported;

@Exported
public class SocketFlow
{
    public static final int UNSET = -1;
    public static final int NORMAL_PRIORITY = 1;
    public static final int HIGH_PRIORITY = 2;
    private static final int NO_STATUS_VALUE = 0;
    private static final int OK_VALUE = 1;
    private static final int NO_PERMISSION_VALUE = 2;
    private static final int NOT_CONNECTED_VALUE = 3;
    private static final int NOT_SUPPORTED_VALUE = 4;
    private static final int ALREADY_CREATED_VALUE = 5;
    private static final int IN_PROGRESS_VALUE = 6;
    private static final int OTHER_VALUE = 7;
    private int priority;
    private long bandwidth;
    private SocketFlow.Status status;
    
    private SocketFlow() {
        this.priority = 1;
        this.bandwidth = -1L;
        this.status = SocketFlow.Status.NO_STATUS;
    }
    
    public static SocketFlow create() {
        return new SocketFlow();
    }
    
    public SocketFlow priority(int priority) {
        if (priority != 1 && priority != 2) {
            throw new IllegalArgumentException("invalid priority");
        }
        this.priority = priority;
        return this;
    }
    
    public SocketFlow bandwidth(long bandwidth) {
        if (bandwidth < 0L) {
            throw new IllegalArgumentException("invalid bandwidth");
        }
        this.bandwidth = bandwidth;
        return this;
    }
    
    public int priority() {
        return this.priority;
    }
    
    public long bandwidth() {
        return this.bandwidth;
    }
    
    public SocketFlow.Status status() {
        return this.status;
    }
    
    void status(int n) {
        this.status = SocketFlow.Status.from(n);
    }
    
    @Exported
    public enum Status
    {
        NO_STATUS(0), 
        OK(1), 
        NO_PERMISSION(2), 
        NOT_CONNECTED(3), 
        NOT_SUPPORTED(4), 
        ALREADY_CREATED(5), 
        IN_PROGRESS(6), 
        OTHER(7);
        
        private final int value;
        
        private Status(int value) {
            this.value = value;
        }
        
        static SocketFlow.Status from(int i) {
            if (i == SocketFlow.Status.NO_STATUS.value) {
                return SocketFlow.Status.NO_STATUS;
            }
            if (i == SocketFlow.Status.OK.value) {
                return SocketFlow.Status.OK;
            }
            if (i == SocketFlow.Status.NO_PERMISSION.value) {
                return SocketFlow.Status.NO_PERMISSION;
            }
            if (i == SocketFlow.Status.NOT_CONNECTED.value) {
                return SocketFlow.Status.NOT_CONNECTED;
            }
            if (i == SocketFlow.Status.NOT_SUPPORTED.value) {
                return SocketFlow.Status.NOT_SUPPORTED;
            }
            if (i == SocketFlow.Status.ALREADY_CREATED.value) {
                return SocketFlow.Status.ALREADY_CREATED;
            }
            if (i == SocketFlow.Status.IN_PROGRESS.value) {
                return SocketFlow.Status.IN_PROGRESS;
            }
            if (i == SocketFlow.Status.OTHER.value) {
                return SocketFlow.Status.OTHER;
            }
            throw new InternalError("Unknown value: " + i);
        }
    }
}
